import math
from adap_gl import AdapGL as agl
from calcroot import Newton

class C_F_tan_kernel:
    def __init__(self, df1, df2):       
        self.df1 = df1
        self.df2 = df2

    def v(self, theta):
        x = math.tan(theta)
        v = pow(x, (self.df1 / 2.0) - 1.0) / \
                pow(1.0 + (self.df1 * x / self.df2), (self.df1 + self.df2) / 2.0)
        return v / (math.cos(theta) ** 2.0)

class C_F_tan_density:
    def __init__(self, df1, df2):
        self.df1 = df1
        self.df2 = df2
        F_kernel =C_F_tan_kernel(self.df1, self.df2)
        gl = agl()
        v = gl.integral(0.0, math.pi/2.0, F_kernel)
        self.c = 1.0 / v

    def v(self, theta):
        x = math.tan(theta)
        v = pow(x, (self.df1 / 2.0) - 1.0) / \
                pow(1.0 + (self.df1 * x / self.df2), (self.df1 + self.df2) / 2.0)
        return self.c * v / (math.cos(theta) ** 2.0)

class C_cum_F:
    def __init__(self, df1, df2):
        self.df1 = df1
        self.df2 = df2
        self.F_tan = C_F_tan_density(self.df1, self.df2)
        self.gl = agl()

    def v(self, F):
        F_cum = self.gl.integral(0.0, math.atan(F), self.F_tan)
        return F_cum

while True:
    try:
        s = input("df1 = ")
        df1 = float(s)
        s = input("df2 = ")
        df2 = float(s)
        s = input("F = ")
        F = float(s)
        cum_F = C_cum_F(df1, df2)
        p = cum_F.v(F)
        print("p = %f   F = %f"%(p, F))
    except:
        break
